from metamart_schemas.generics import PackageConfig


class Config(PackageConfig):
    """ """

    integration_name = "metamart-source-flat-file"
    metadata_id = "metamart_source_flat_file"


config = Config()



import os
from itertools import chain
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union

import pandas as pd

from metamart_source_flat_file.models import ID, Column, Edge, Table

LOADER_MAP = {".csv": pd.read_csv, ".parquet": pd.read_parquet, ".feather": pd.read_feather}


def load_file(file_name: str, file_ext: str) -> pd.DataFrame:
    """

    Args:
        file_name: The path to the file
        file_ext: The type of file

    Returns:

    Raises:

    """
    assert file_ext in LOADER_MAP, f"{file_ext} not supported. Choose one of {set(LOADER_MAP.keys())}"
    return LOADER_MAP[file_ext](file_name)


def map_pandas_types(dtype) -> str:
    """

    Args:
        dtype:

    Returns:

    Raises:

    """
    dtype = str(dtype).lower()
    if dtype.startswith("int"):
        return "integer"
    elif dtype.startswith("float"):
        return "float"
    elif dtype.startswith("object"):
        return "string"
    else:
        return dtype


def build_column(data: pd.Series, namespace: str, table_name: str) -> Column:
    """

    Args:
        data (pd.Series):
        namespace (str):
        table_name (str):

    Returns:

    Raises:

    """
    metadata = {
        "name": data.name,
        "namespace": namespace,
        "table": table_name,
        "data_type": map_pandas_types(data.dtype),
        "is_nullable": data.hasnans,
    }
    return Column(**metadata)


def column_builder(namespace: str, table_name: str) -> Callable[[pd.Series], Column]:
    """

    Args:
        namespace (str):
        table_name (str):

    Returns:

    Raises:

    """

    def inner(data: pd.Series) -> Column:
        """

        Args:
            data (pd.Series):

        Returns:

        Raises:

        """
        return build_column(data, namespace, table_name)

    return inner


def table_builder(namespace: str, table_name: str, file_location: str) -> Table:
    """

    Args:
        namespace (str):
        table_name (str):
        file_location (str):

    Returns:

    Raises:

    """
    return Table(namespace=namespace, file_name=file_location, name=table_name)


def build_nodes_and_edges(
    file_ref: str, file_type: str, table_name: str, file_location: str, namespace: str
) -> Tuple[List[Union[Table, Column]], List[Edge]]:
    """

    Args:
        file_ref:
        file_type:
        table_name:
        file_location:
        namespace (str):

    Returns:

    Raises:

    """
    df = load_file(file_ref, file_type)

    builder = column_builder(namespace, table_name)
    columns = [builder(df[col]) for col in df.columns]

    table = table_builder(namespace, table_name, file_location)
    table.columns = columns

    nodes = [table, *columns]
    return nodes, table.get_edges()



import os
from functools import cache
from typing import List, Optional, Tuple

from metamart_schemas.base import SourcedEdge, SourcedNode
from metamart_schemas.integrations.base import MetamartIntegrationImplementation
from metamart_schemas.v1.source import SourceV1

from metamart_source_flat_file.adapters import adapt_to_client
from metamart_source_flat_file.loader import LOADER_MAP, build_nodes_and_edges


class FlatFileIntegration(MetamartIntegrationImplementation):
    """A class for extracting Metamart compliant metadata from flat files like csv and parquet.

    Attributes:
        file_name: A path to the file
        namespace: The Metamart namespace to associate with output from the integration

    """

    def __init__(
        self,
        file_ref: str,
        namespace: str,
        source: SourceV1,
        file_ext: Optional[str] = None,
        table_name: Optional[str] = None,
        file_location: Optional[str] = None,
        version: Optional[str] = None,
    ):
        """Initializes the Flat File integration.

        Args:
            file_ref: A loadable file reference
            namespace: The Metamart namespace to associate with output from the integration
            source: The Metamart data source to associate with output from the integration. More information about source objects is available in the `metamart_schemas` library.
            version: The Metamart data version to associate with output from the integration
        """
        super().__init__(source, version)
        try:
            default_table_name, default_file_ext = os.path.splitext(file_ref)
        except:
            default_table_name, default_file_ext = None, None

        self.file_ref = file_ref
        if file_ext is not None:
            self.file_ext = file_ext
        elif default_file_ext is not None:
            self.file_ext = default_file_ext
        else:
            message = (
                f"Could not extract the file type from the provided file object of type `{type(self.file_ref)}`. "
                "You must either provide the file_ref as a string or provide an explicit `file_ext` value."
            )
            raise ValueError(message)

        if table_name is not None:
            self.table_name = table_name
        elif default_table_name is not None:
            self.table_name = default_table_name
        else:
            message = (
                f"Could not extract a table name from the provided file object of type `{type(self.file_ref)}`. "
                "You must either provide the file_ref as a string or provide an explicit `table_name` value."
            )
            raise ValueError(message)

        self.file_location = file_location
        if self.file_location is None and isinstance(self.file_ref, str):
            self.file_location = self.file_ref

        self.namespace = namespace

    @cache
    def get_nodes_and_edges(self) -> Tuple[List[SourcedNode], List[SourcedEdge]]:
        """Returns a tuple of lists of SourcedNode and SourcedEdge objects"""
        nodes, edges = build_nodes_and_edges(
            self.file_ref, self.file_ext, self.table_name, self.file_location, self.namespace
        )
        nodes = adapt_to_client(nodes, self.source, self.version)
        edges = adapt_to_client(edges, self.source, self.version)
        return nodes, edges

    def nodes(self) -> List[SourcedNode]:
        """Returns a list of SourcedNode objects"""
        return self.get_nodes_and_edges()[0]

    def edges(self) -> List[SourcedEdge]:
        """Returns a list of SourcedEdge objects"""
        return self.get_nodes_and_edges()[1]

    def ready(self) -> bool:
        """Returns True if the integration is ready to run"""
        return self.file_ext in LOADER_MAP



from typing import List, Literal, Optional, Union

from pydantic import BaseModel, Field


class ID(BaseModel):
    """ """

    name: str
    namespace: str
    # full_name: str


class Table(ID):
    """ """

    file_name: str
    columns: Optional[List["Column"]] = None

    @property
    def full_name(self):
        """ """
        return self.name

    def get_edges(self) -> List["Edge"]:
        """

        Args:

        Returns:

        Raises:

        """
        if self.columns is None:
            return []

        return [Edge(source=self, destination=column) for column in self.columns]


class Column(ID):
    """ """

    name: str = Field(alias="column_name")
    namespace: str
    table: str
    data_type: str
    is_nullable: bool

    class Config:
        """ """

        allow_population_by_field_name = True

    @property
    def full_name(self):
        """ """
        return f"{self.table}.{self.name}"


class Edge(BaseModel):
    """ """

    source: Union[Table, Column]
    destination: Union[Table, Column]
    constraint_type: Literal["bt"] = "bt"
